#
# spec file for package metis
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor serial%{nil}

%define vers 5.1.0
%define _vers 5_1_0
%define short_ver 5.1
%define src_ver %{version}
%define pname metis
%define somajor 5

%if "%{flavor}" == ""
ExclusiveArch:  do_not_build
%define package_name %pname
%bcond_with hpc
%endif

%if "%{flavor}" == "serial"
%bcond_with hpc
%endif

%if "%{flavor}" == "gnu-hpc"
%bcond_without hpc
%global compiler_family gnu
%undefine c_f_ver
%endif

%if "%{flavor}" == "gnu6-hpc"
%bcond_without hpc
%define compiler_family gnu
%define c_f_ver 6
%undefine mpi_flavor
%endif

%if "%{flavor}" == "gnu7-hpc"
%bcond_without hpc
%define compiler_family gnu
%define c_f_ver 7
%undefine mpi_flavor
%endif

%if "%{flavor}" == "gnu8-hpc"
%bcond_without hpc
%define compiler_family gnu
%define c_f_ver 8
%undefine mpi_flavor
%endif

%if "%{flavor}" == "gnu9-hpc"
%bcond_without hpc
%define compiler_family gnu
%define c_f_ver 9
%undefine mpi_flavor
%endif

%if "%{flavor}" == "gnu10-hpc"
%bcond_without hpc
%define compiler_family gnu
%define c_f_ver 10
%undefine mpi_flavor
%endif

%if %{without hpc}
%if 0%{!?package_name:1}
%define package_name  %{pname}
%endif
%define p_prefix %_prefix
%define p_includedir %_includedir
%define p_libdir %_libdir
%define p_bindir %_bindir
%define p_mandir %_mandir
%define p_datadir %_datadir
%define p_infodir %_infodir
%define libname lib%{pname}%{somajor}

%else

%define package_name %{hpc_package_name %_vers}
%define p_prefix %hpc_prefix
%define p_includedir %hpc_includedir
%define p_libdir %hpc_libdir
%define p_bindir %hpc_bindir
%define p_mandir %hpc_mandir
%define p_datadir %hpc_datadir
%define p_infodir %hpc_infodir
%define libname lib%{pname}%{hpc_package_name_tail %{_vers}}

%{hpc_init -c %{compiler_family} %{?c_f_ver:-v %{c_f_ver}} %{?ext:-e %{ext}}}
%endif

Name:           %{package_name}
Version:        %{vers}
Release:        11.2
Summary:        Serial Graph Partitioning and Fill-reducing Matrix Ordering
License:        Apache-2.0
Group:          Productivity/Scientific/Math
URL:            http://glaros.dtc.umn.edu/gkhome/metis/metis/overview
Source0:        http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/%{pname}-%{version}.tar.gz
# PATCH-FIX-OPENSUSE metis-cmake.patch
Patch1:         metis-cmake.patch
# PATCH-FIX-OPENSUSE metis-programs-no-compilation-time.patch -- Fix W: file-contains-date-and-time
Patch2:         metis-programs-no-compilation-time.patch
Patch3:         metis-makefile-c-directives.patch
BuildRequires:  cmake >= 2.8
BuildRequires:  gcc
BuildRequires:  gcc-c++
%if %{with hpc}
BuildRequires:  %{compiler_family}%{?c_f_ver}-compilers-hpc-macros-devel
BuildRequires:  lua-lmod
BuildRequires:  suse-hpc
%hpc_requires
Requires:       %libname = %version
%endif

Recommends:     %{name}-doc = %{version}
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
METIS is a family of programs for partitioning unstructured graphs and hypergraph
and computing fill-reducing orderings of sparse matrices. The underlying algorithms
used by METIS are based on a multilevel paradigm that, at the time, had been
shown to produce quality results and scale to large problems.

%{?with_hpc:%{hpc_master_package -L}}

%package     -n %{libname}
Summary:        Serial Graph Partitioning and Fill-reducing Matrix Ordering library
Group:          System/Libraries
Obsoletes:      %libname < %{version}

%description -n %{libname}
METIS library provides to partitioning unstructured graphs and hypergraph
and computing fill-reducing orderings of sparse matrices. The underlying algorithms
used by METIS are based on a multilevel paradigm that, at the time, had been
shown to produce quality results and scale to large problems.

%{?with_hpc:%{hpc_master_package -l -L}}

%package        devel
Summary:        Metis development files
Group:          Development/Libraries/C and C++
Requires:       %libname = %{version}
%if %{with hpc}
%hpc_requires_devel
Requires:       %libname = %version
%endif

%description    devel
METIS library provides to partitioning unstructured graphs and hypergraph
and computing fill-reducing orderings of sparse matrices. The underlying algorithms
used by METIS are based on a multilevel paradigm that, at the time, had been
shown to produce quality results and scale to large problems.

This package provides development files.

%{?with_hpc:%{hpc_master_package -L devel}}

%package        doc
Summary:        Metis documentation
Group:          Documentation/Other
BuildArch:      noarch

%description    doc
METIS is a family of programs for partitioning unstructured graphs and hypergraph
and computing fill-reducing orderings of sparse matrices. The underlying algorithms
used by METIS are based on a multilevel paradigm that, at the time, had been
shown to produce quality results and scale to large problems.

%{?with_hpc:%{hpc_master_package doc}}

%package        examples
Summary:        Metis examples
Group:          Documentation/Other
BuildArch:      noarch

%description    examples
METIS library provides to partitioning unstructured graphs and hypergraph
and computing fill-reducing orderings of sparse matrices. This package provides
graph files you can use to test Metis.

%{?with_hpc:%{hpc_master_package examples}}

%prep
%{?with_hpc: %hpc_debug}
%autosetup -p1 -n %{pname}-%{version}

# set width (32 or 64 bits) of the elementary data type, see Install.txt
sed -i 's|#define IDXTYPEWIDTH 32|#define IDXTYPEWIDTH %{__isa_bits}|' include/metis.h

%build

%if %{with hpc}
%hpc_debug
%hpc_setup_compiler
%endif

make config shared=1 prefix=%{p_prefix} cflags="%{optflags} -fopenmp -pthread -fpie -pie" ldflags="-pie"
make %{?_smp_mflags}

%install
%{?with_hpc:%hpc_setup}
%{?with_hpc:%hpc_debug}
make install DESTDIR=%{buildroot}

%if %{with hpc}
%{hpc_write_pkgconfig}

%hpc_write_modules_files
#%%Module1.0#####################################################################

proc ModulesHelp { } {

puts stderr " "
puts stderr "This module loads the %{pname} library built with the %{compiler_family} compiler toolchain."
puts stderr "\nVersion %{version}\n"

}
module-whatis "Name: %{pname} built with %{compiler_family} toolchain"
module-whatis "Version: %{version}"
module-whatis "Category: runtime library"
module-whatis "Description: %{SUMMARY}"
module-whatis "%{url}"

set     version                     %{version}

prepend-path    PATH                %{hpc_bindir}
prepend-path    LD_LIBRARY_PATH     %{hpc_libdir}

setenv          %{hpc_upcase %pname}_DIR        %{hpc_prefix}
setenv          %{hpc_upcase %pname}_BIN        %{hpc_bindir}
setenv          %{hpc_upcase %pname}_LIB        %{hpc_libdir}

prepend-path    LIBRARY_PATH        %{hpc_libdir}
if {[file isdirectory  %{hpc_includedir}]} {
prepend-path    CPATH               %{hpc_includedir}
prepend-path    C_INCLUDE_PATH      %{hpc_includedir}
prepend-path    CPLUS_INCLUDE_PATH  %{hpc_includedir}
prepend-path    INCLUDE             %{hpc_includedir}
%hpc_modulefile_add_pkgconfig_path
setenv          %{hpc_upcase %pname}_INC        %{hpc_includedir}
}

family "%pname"

EOF
%endif

%check
pushd graphs
LD_LIBRARY_PATH=%{buildroot}%{p_libdir}:$LD_LIBRARY_PATH %{buildroot}%{p_bindir}/ndmetis mdual.graph
LD_LIBRARY_PATH=%{buildroot}%{p_libdir}:$LD_LIBRARY_PATH %{buildroot}%{p_bindir}/mpmetis metis.mesh 2
LD_LIBRARY_PATH=%{buildroot}%{p_libdir}:$LD_LIBRARY_PATH %{buildroot}%{p_bindir}/gpmetis test.mgraph 4
LD_LIBRARY_PATH=%{buildroot}%{p_libdir}:$LD_LIBRARY_PATH %{buildroot}%{p_bindir}/gpmetis copter2.graph 4
LD_LIBRARY_PATH=%{buildroot}%{p_libdir}:$LD_LIBRARY_PATH %{buildroot}%{p_bindir}/graphchk 4elt.graph
popd
cd graphs # cleanup after tests:
rm test.mgraph.part.4 metis.mesh.epart.2 metis.mesh.npart.2 copter2.graph.part.4 mdual.graph.iperm

%if %{without hpc}
%post -n %libname -p /sbin/ldconfig

%postun -n %libname -p /sbin/ldconfig
%else
%post -n %libname
/sbin/ldconfig -N %{p_libdir}

%postun -n %libname
/sbin/ldconfig -N %{p_libdir}
%hpc_module_delete_if_default
%endif

%files
%{?with_hpc:%dir %p_bindir}
%doc Changelog
%license LICENSE.txt
%{p_bindir}/*

%files -n %{libname}
%{?with_hpc:%hpc_dirs}
%{?hpc_modules_files}
%{p_libdir}/lib%{pname}.so.%{somajor}*

%files devel
%{?with_hpc:%dir %{p_includedir}}
%{p_includedir}/%{pname}.h
%{p_libdir}/lib%{pname}.so
%{?with_hpc:%{hpc_pkgconfig_file -n %{pname}}}

%files doc
%doc manual/manual.pdf

%files examples
%doc graphs

%changelog
* Wed Apr  3 2024 Bernhard Wiedemann <bwiedemann@suse.com>
- Cleanup after tests
* Fri Feb 23 2024 pgajdos@suse.com
- Use %%autosetup macro. Allows to eliminate the usage of deprecated
  %%patchN
* Thu Jul 23 2020 Egbert Eich <eich@suse.com>
- Add build support for gcc10 to HPC build (bsc#1174439).
* Wed Feb 26 2020 Egbert Eich <eich@suse.com>
- Add support for gcc8/9 building (jsc#SLE-8604).
* Fri Mar 15 2019 Egbert Eich <eich@suse.com>
- Build HPC master package for 'examples'.
* Tue Feb 12 2019 Antoine Ginies <aginies@suse.com>
- add a examples subpackage which include graphs* file to test Metis
* Mon Feb 11 2019 Egbert Eich <eich@suse.com>
- Set default module version correctly when installing
  master package, unset when deinstalling the default library
  package.
- Fix %%%%post and %%%%postun scripts for HPC.
- Fix dependencies for HPC.
- Fix HPC modulefile:
  * Aibraries are always there when module file is installed.
  * Set PKG_CONFIG_PATH.
- Fix package group names.
* Mon Jan 14 2019 eich@suse.com
- Remove dependency to a non-existing module package for HPC.
* Thu Nov 22 2018 Jan Engelhardt <jengelh@inai.de>
- Edit description to put time-sensitive wording into context.
* Fri Oct 26 2018 eich@suse.com
- General spec file clean up.
- Touch-up to the HPC build.
* Thu Nov 30 2017 jjolly@suse.com
- Implemented suse-hpc packaging
- Added metis-makefile-c-directives.patch
  - Provides cflags option to help provide metis native build process
* Fri Jun 20 2014 dmitry_r@opensuse.org
- Update to version 5.1.0, changes since version 4.0
  * strtoidx() addition for portable input file parsing
  * Fixed some bad memory allocation calls
  * Incorporated GKlib into METIS, which replaced many of its
    internal functions.
  * A nearly complete re-write of Metis' code-based that changed
    expanded the functionality of the command-line programs and
    API routines
  * Multi-constraint partitioning can be used in conjunction with
    minimization of the total communication volume.
  * All graph and mesh partitioning routines take as input the
    target sizes of the partitions, which among others, allow them
    to compute partitioning solutions that are well-suited for
    parallel architectures with heterogeneous computing capabilities.
  * When multi-constraint partitioning is used, the target sizes of
    the partitions are specified on a per partition-constraint pair.
  * The multilevel k-way partitioning algorithms can compute a
    partitioning solution in which each partition is contiguous.
  * All partitioning and ordering routines can compute multiple
    different solutions and select the best as the final solution.
  * The mesh partitioning and mesh-to-graph conversion routines can
    operate on mixed element meshes.
  * The command-line programs provide full access to the entire
    set of capabilities provided by Metis' API.
  * Re-written the memory management subsystem to reduce overall
    memory requirements.
  * Added back Fortran support in the form of alternate API names
    (see libmetis/frename.h).
  * Changed the license to Apache Version 2
- Move metis shared library to separate package
- Enable build tests
- Use proper build parameters
  * metis-cmake.patch
- Delete compilation time from metis programs to avoid unnecessary
  package republishing
  * metis-programs-no-compilation-time.patch
* Fri Aug 17 2012 scorot@free.fr
- Buildreqs libtool for 12.2 an higher
- spec file reformating
* Sat Mar 17 2012 scorot@free.fr
- remove %%%%debug_package macro which broke build on 11.3
* Sun Nov  8 2009 andrea@opensuse.org
- new package
